"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Description:
Tests for module: extraction_tools_cache.osrs_cache_data module

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""

import pytest
from pathlib import Path

from extraction_tools_cache import osrs_cache_data


@pytest.mark.parametrize("test_data,expected", [
    ({"id": 7410, "name": "Greater abyssal demon", "models": [32921]},
     "eJyrVspMUbJSMDcxNNBRUMpLzE0F8pTci1ITS1KLFBKTKouLE3MUUlJz8/OUgApy81NSc4qBSqKNjSyNDGNrAWB5EzI=")
])
def test_osrs_cache_data_compression(test_data, expected):
    id_number, json_out = osrs_cache_data.compress_definition_file(test_data)
    assert json_out == expected


@pytest.mark.parametrize("cache_type,expected", [
    ("items", 23122),
    ("npcs", 8645),
    ("objects", 34714)
])
def test_osrs_cache_data_decompression(path_to_cache_dir: Path, cache_type, expected):
    path_to_cache_file = path_to_cache_dir / f"{cache_type}.json"
    definitions = osrs_cache_data.CacheDefinitionFiles(path_to_cache_file)
    definitions.decompress_cache_file()
    assert len(definitions) == expected



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""

import os
import json
import glob
from pathlib import Path
from typing import Dict
from typing import List
from typing import Generator

from osrsbox.items_api.item_definition import ItemDefinition

PATH_TO_DOCS = Path(__file__).absolute().parent / ".." / ".." / "docs"
PATH_TO_ITEMS_COMPLETE_JSON = PATH_TO_DOCS / "items-complete.json"


class AllItems:
    """This class handles loading of the osrsbox-db items database.

    :param input_data_file_or_directory: The osrsbox-db items folder of JSON files, or single JSON file.
    """
    def __init__(self, input_data_file_or_directory: str = PATH_TO_ITEMS_COMPLETE_JSON):
        self.all_items: List[ItemDefinition] = list()
        self.all_items_dict: Dict[int, ItemDefinition] = dict()
        self.load_all_items(input_data_file_or_directory)

    def __iter__(self) -> Generator[str, None, None]:
        """Iterate (loop) over each ItemDefinition object."""
        for item_id in self.all_items:
            yield item_id

    def __getitem__(self, id_number: int) -> ItemDefinition:
        """Return the item definition object for a loaded item.

        :param id_number: The item ID number.
        :return: The item definition object linked to a specific ID number.
        """
        return self.all_items_dict[id_number]

    def __len__(self) -> int:
        """Return the count of the total number of items.

        :return: The total number of items.
        """
        return len(self.all_items)

    def load_all_items(self, input_data_file_or_directory: str) -> None:
        """Load the items database via a JSON file, or directory of JSON files.

        :param input_data_file_or_directory: The path to the data input.
        """
        if os.path.isdir(input_data_file_or_directory):

            if input_data_file_or_directory.endswith("/"):
                path = input_data_file_or_directory + "*"
            else:
                path = os.path.join(input_data_file_or_directory, "*")

            self._load_items_from_directory(path_to_directory=path)

        elif os.path.isfile(input_data_file_or_directory):
            self._load_items_from_file(input_data_file_or_directory)

        # Sort the list of items
        self.all_items.sort(key=lambda x: x.id)

    def _load_items_from_directory(self, path_to_directory: str) -> None:
        """Load item database from a directory of JSON files (`items-json`).

        :param path_to_directory: The path to the `items-json` directory.
        """
        # Loop through every item file
        for json_file in glob.glob(path_to_directory):

            if os.path.isdir(json_file):
                continue

            with open(json_file) as input_json_file:
                temp = json.load(input_json_file)

            self._load_item(temp)

    def _load_items_from_file(self, path_to_json_file: str) -> None:
        """Load item database from a single JSON file (`items-complete.json`).

        :param path_to_json_file: The path to the `items-complete.json` file.
        """
        with open(path_to_json_file) as input_json_file:
            temp = json.load(input_json_file)

        for entry in temp:
            self._load_item(temp[entry])

    def _load_item(self, item_json: Dict) -> None:
        """Convert the `item_json` into a :class:`ItemDefinition` and store it."""
        # Load the item using the ItemDefinition class
        item_def = ItemDefinition(**item_json)

        # Add item to list
        self.all_items.append(item_def)
        self.all_items_dict[item_def.id] = item_def



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""

from typing import Dict


class ItemEquipment:
    """This class defines the properties for an equipable OSRS item.

    The ItemEquipment class is the object that retains all items properties related
    to equipable items. This includes item stats (attack, defence bonuses) and
    additional properties about equipment slot, attack speed and skill requirements
    for the item.
    """

    def __init__(self, attack_stab=None, attack_slash=None, attack_crush=None, attack_magic=None,
                 attack_ranged=None, defence_stab=None, defence_slash=None, defence_crush=None,
                 defence_magic=None, defence_ranged=None, melee_strength=None, ranged_strength=None,
                 magic_damage=None, prayer=None, slot=None, attack_speed=None, skill_reqs=None):
        self.attack_stab = attack_stab
        self.attack_slash = attack_slash
        self.attack_crush = attack_crush
        self.attack_magic = attack_magic
        self.attack_ranged = attack_ranged
        self.defence_stab = defence_stab
        self.defence_slash = defence_slash
        self.defence_crush = defence_crush
        self.defence_magic = defence_magic
        self.defence_ranged = defence_ranged
        self.melee_strength = melee_strength
        self.ranged_strength = ranged_strength
        self.magic_damage = magic_damage
        self.prayer = prayer
        self.slot = slot
        self.attack_speed = attack_speed
        self.skill_reqs = skill_reqs

    def construct_json(self) -> Dict:
        """Construct dictionary/JSON of item_equipment property for exporting or printing.

        :return json_out: A dictionary of all equipment properties.
        """
        json_out = dict()
        for prop in self.__dict__:
            json_out[prop] = getattr(self, prop)

        return json_out
